#include <reg552.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rtx51.h>

#define  MASK    0x07

#define  SYSTEM  0
#define  SERIAL  1
#define	 PHASE	 2

typedef enum {NO_OK=0, YES_OK=1, DIR_R=2, DIR_L=3} Chyby;
typedef enum {OFF=0, ON=1} Stavy;
typedef enum {R=0, L=1} Smery;

char DIR;              //Vysledok zistenia faz ...
sbit ERR  = P3^2;      //ERR=0 je chyba, ERR=1 je OK
sbit LEDR = P3^3;      //LED signalizujuca otacanie sa faz RIGHT
sbit LEDL = P3^4;      //LED signalizujuca otacanie sa faz LEFT

                  //123456   123456
code	char *Smer[]={"645132","623154"};
code	char *Message[]={"NOT_OK","OK","RIGHT","LEFT"};
idata	char Poradie_X[12];

unsigned char Port(unsigned char Maska) 
{
 code char Tab[]="01234567"; //Najlepsie je to namapovat na prve porty P1, P3 .... ine by boli komplikovanejsie ...
 return Tab[(P1&Maska)];
}

void Serial(void) _task_ SERIAL _priority_ 0
{
 signed char RtxCompletion;
 PCON|=0x80;
 S0CON=0x50;
 TMOD|=0x20;
 TH1=TL1=0xFB;
 TR1=1;
 TI=1;
 printf("System Ready ...\r\n");
 while(1)
  {
   switch(RtxCompletion=os_wait(K_TMO+K_SIG,250,NULL))
    {
     case TMO_EVENT : printf("Smer otacania faz = %s\r\n",Message[DIR]);break;
     case SIG_EVENT : printf("Chyba %02bd\r\n",DIR); break;
    }
  }
}

#define	Timeout	30																			//Timeout na Error-y

#pragma REGISTERBANK(0)

void Phase(void) _task_ PHASE _priority_ 2
{
 signed char RtxCompletion;
 unsigned char i=0; 
 TM2CON=0x01;
 CTCON|=0x20+0x10+0x08+0x04+0x02+0x01;
 ERR=YES_OK;
 RtxCompletion=os_attach_interrupt(6);
 RtxCompletion=os_attach_interrupt(7);
 RtxCompletion=os_attach_interrupt(8);
 while(1)
  {
   switch(RtxCompletion=os_wait(K_INT+K_TMO+K_SIG,Timeout,NULL))			//Cakam na fazu A,B,C a zachytavam obsahy zachytnych registrov ktore vyjadruju dlzku 1/2 periody fazoveho napatia siete
    {
     case	INT_EVENT	:	switch(TM2IR&0x07)
											 {
											  case	0x01	:	CTI0=0; break;
												case	0x02	:	CTI1=0; break;
												case	0x04	:	CTI2=0; break;
												default			: TM2IR&=~0x07; break;						//Tu by som sa nemal dostat ak spracujem tasky zmeny urovni nacas ....
											 }
											if(i>=sizeof(Poradie_X)) 
											 {
											  i=0x00;
												RtxCompletion=os_send_signal(SYSTEM);
											 }
											Poradie_X[i++]=Port(MASK); break; 							//Prisla hrana fazy, tak vykonam toto ...
	   case	TMO_EVENT	:	RtxCompletion=os_send_signal(PHASE); break;		//Ak sa tu dostanem neprisla hrana fazy do TIME tak vyvolam chybu ...
     case SIG_EVENT : ERR=NO_OK;
                      DIR=NO_OK; 
                      RtxCompletion=os_send_signal(SERIAL);
                      break;
	  }
  }
}

#pragma REGISTERBANK(0)

void System(void) _task_ SYSTEM _priority_ 0
{
 signed char RtxCompletion;
 RtxCompletion=os_set_slice(1536);
 RtxCompletion=os_wait(K_TMO,20,NULL);
 #ifdef RS232
 RtxCompletion=os_create_task(SERIAL); 
 #endif
 RtxCompletion=os_create_task(PHASE);
 while(1)
  {
	 RtxCompletion=os_wait(K_SIG,0xFF,NULL);
   if((strstr(Poradie_X,Smer[R]))!=NULL) DIR=DIR_L; 	 //Smer otacania sa faz je vlavo
   if((strstr(Poradie_X,Smer[L]))!=NULL) DIR=DIR_R; 	 //Smer otacania sa faz je vpravo
   switch(DIR)                                         //Nastavim OK, alebo ERROR
    {
     case DIR_R : LEDR=ON; LEDL=OFF; ERR=YES_OK; break;
     case DIR_L : LEDL=ON; LEDR=OFF; ERR=YES_OK; break;
     default    : LEDL=LEDR=OFF; ERR=NO_OK; break;
    }
  }
}

void main(void)
{
 signed char RtxCompletion;
 RtxCompletion=os_start_system(SYSTEM);
}